@@ -9,9 +9,7 @@ use DBI;
 use Class::ReturnValue;
 use Encode qw();
 
-use DBIx::SearchBuilder::Util qw/ sorted_values /;
-
-use vars qw(@ISA %DBIHandle $PrevHandle $DEBUG %TRANSDEPTH %TRANSROLLBACK %FIELDS_IN_TABLE);
+use vars qw(@ISA %DBIHandle $PrevHandle $DEBUG %TRANSDEPTH %FIELDS_IN_TABLE);
 
 
 =head1 NAME
@@ -359,7 +357,7 @@ Splits the key value pairs, constructs an INSERT statement
 and returns query string and set of bind values.
 
 This method is more useful for subclassing in DB specific
-handles. L</Insert> method is preferred for end users.
+handles. L</Insert> method is prefered for end users.
 
 =cut
 
@@ -443,7 +441,7 @@ sub UpdateRecordValue {
 
   ## Constructs the where clause.
   my $where  = 'WHERE ';
-  foreach my $key (sort keys %{$args{'PrimaryKeys'}}) {
+  foreach my $key (keys %{$args{'PrimaryKeys'}}) {
      $where .= $key . "=?" . " AND ";
      push (@bind, $args{'PrimaryKeys'}{$key});
   }
@@ -496,15 +494,15 @@ sub SimpleUpdateFromSelect {
     my ($self, $table, $values, $query, @query_binds) = @_;
 
     my @columns; my @binds;
-    for my $k (sort keys %$values) {
+    while ( my ($k, $v) = each %$values ) {
         push @columns, $k;
-        push @binds, $values->{$k};
+        push @binds, $v;
     }
 
     my $full_query = "UPDATE $table SET ";
     $full_query .= join ', ', map "$_ = ?", @columns;
     $full_query .= ' WHERE id IN ('. $query .')';
-    my $sth = $self->SimpleQuery( $full_query, @binds, @query_binds );
+    my $sth = $self->SimpleQuery( $full_query, @binds );
     return $sth unless $sth;
 
     my $rows = $sth->rows;
@@ -841,23 +839,15 @@ sub EndTransaction {
     $depth = 0 if $args{'Force'};
 
     $self->TransactionDepth( $depth );
-
-    my $dbh = $self->dbh;
-    $TRANSROLLBACK{ $dbh }{ $action }++;
-    if ( $TRANSROLLBACK{ $dbh }{ $action eq 'commit'? 'rollback' : 'commit' } ) {
-        warn "Rollback and commit are mixed while escaping nested transaction";
-    }
     return 1 if $depth;
 
-    delete $TRANSROLLBACK{ $dbh };
-
     if ($action eq 'commit') {
-        return $dbh->commit;
+        return $self->dbh->commit;
     }
     else {
         DBIx::SearchBuilder::Record::Cachable->FlushCache
             if DBIx::SearchBuilder::Record::Cachable->can('FlushCache');
-        return $dbh->rollback;
+        return $self->dbh->rollback;
     }
 }
 
@@ -1032,9 +1022,6 @@ sub Join {
                 }
             }
 
-        } else {
-            # we found alias, so NewAlias should take care of distinctness
-            $args{'DISTINCT'} = 1 unless exists $args{'DISTINCT'};
         }
 
         unless ( $alias ) {
@@ -1056,7 +1043,7 @@ sub Join {
         push @{$args{SearchBuilder}{aliases}}, @{$collection->{aliases}};
 
         # Move over joins, as well
-        for my $join (sort keys %{$collection->{left_joins}}) {
+        for my $join (keys %{$collection->{left_joins}}) {
             my %alias = %{$collection->{left_joins}{$join}};
             $alias{depends_on} = $alias if $alias{depends_on} eq "main";
             $alias{criteria} = $self->_RenameRestriction(
@@ -1090,12 +1077,6 @@ sub Join {
     $meta->{'criteria'}{'base_criterion'} =
         [ { field => "$alias.$args{'FIELD2'}", op => '=', value => $criterion } ];
 
-    if ( $args{'DISTINCT'} && !defined $args{'SearchBuilder'}{'joins_are_distinct'} ) {
-        $args{SearchBuilder}{joins_are_distinct} = 1;
-    } elsif ( !$args{'DISTINCT'} ) {
-        $args{SearchBuilder}{joins_are_distinct} = 0;
-    }
-
     return ($alias);
 }
 
@@ -1188,7 +1169,7 @@ sub _BuildJoins {
     while ( my @list =
         grep !$processed{ $_ }
             && (!$joins->{ $_ }{'depends_on'} || $processed{ $joins->{ $_ }{'depends_on'} }),
-        sort keys %$joins
+        keys %$joins
     ) {
         foreach my $join ( @list ) {
             $processed{ $join }++;
@@ -1202,7 +1183,7 @@ sub _BuildJoins {
                         $_->{'field'} .' '. $_->{'op'} .' '. $_->{'value'}:
                         $_
                 }
-                map { ('(', @$_, ')', $aggregator) } sorted_values($meta->{'criteria'});
+                map { ('(', @$_, ')', $aggregator) } values %{ $meta->{'criteria'} };
             pop @tmp;
             $join_clause .= join ' ', @tmp;
         }
@@ -1232,7 +1213,7 @@ sub OptimizeJoins {
     # finally we'll get ordered list with leafes in the beginning and top most nodes at
     # the end.
     while ( my @list = grep !$processed{ $_ }
-            && $processed{ $joins->{ $_ }{'depends_on'} }, sort keys %$joins )
+            && $processed{ $joins->{ $_ }{'depends_on'} }, keys %$joins )
     {
         unshift @ordered, @list;
         $processed{ $_ }++ foreach @list;
@@ -1272,19 +1253,19 @@ sub MayBeNull {
 
     # build full list of generic conditions
     my @conditions;
-    foreach ( grep @$_, sorted_values($args{'SearchBuilder'}->{'restrictions'}) ) {
+    foreach ( grep @$_, values %{ $args{'SearchBuilder'}->{'restrictions'} } ) {
         push @conditions, 'AND' if @conditions;
         push @conditions, '(', @$_, ')';
     }
 
     # find tables that depends on this alias and add their join conditions
-    foreach my $join ( sorted_values($args{'SearchBuilder'}->{'left_joins'}) ) {
+    foreach my $join ( values %{ $args{'SearchBuilder'}->{'left_joins'} } ) {
         # left joins on the left side so later we'll get 1 AND x expression
         # which equal to x, so we just skip it
         next if $join->{'type'} eq 'LEFT';
         next unless $join->{'depends_on'} eq $args{'ALIAS'};
 
-        my @tmp = map { ('(', @$_, ')', $join->{'entry_aggregator'}) } sorted_values($join->{'criteria'});
+        my @tmp = map { ('(', @$_, ')', $join->{'entry_aggregator'}) } values %{ $join->{'criteria'} };
         pop @tmp;
 
         @conditions = ('(', @conditions, ')', 'AND', '(', @tmp ,')');
@@ -1428,19 +1409,18 @@ sub DistinctCount {
 
 sub Fields {
     my $self  = shift;
-    my $table = lc shift;
+    my $table = shift;
 
-    unless ( $FIELDS_IN_TABLE{$table} ) {
-        $FIELDS_IN_TABLE{ $table } = [];
-        my $sth = $self->dbh->column_info( undef, '', $table, '%' )
+    unless ( keys %FIELDS_IN_TABLE ) {
+        my $sth = $self->dbh->column_info( undef, '', '%', '%' )
             or return ();
         my $info = $sth->fetchall_arrayref({});
         foreach my $e ( @$info ) {
-            push @{ $FIELDS_IN_TABLE{ $table } }, lc $e->{'COLUMN_NAME'};
+            push @{ $FIELDS_IN_TABLE{ lc $e->{'TABLE_NAME'} } ||= [] }, lc $e->{'COLUMN_NAME'};
         }
     }
 
-    return @{ $FIELDS_IN_TABLE{ $table } };
+    return @{ $FIELDS_IN_TABLE{ lc $table } || [] };
 }
 
 
@@ -1558,7 +1538,7 @@ sub DateTimeFunction {
     my $self = shift;
     my %args = (
         Field => undef,
-        Type => '',
+        Type => undef,
         Timezone => undef,
         @_
     );
@@ -1613,109 +1593,6 @@ sub ConvertTimezoneFunction {
     return $args{'Field'};
 }
 
-=head2 DateTimeIntervalFunction
-
-Generates a function to calculate interval in seconds between two
-dates. Takes From and To arguments which can be either scalar or
-a hash. Hash is processed with L<DBIx::SearchBuilder/CombineFunctionWithField>.
-
-Arguments are not quoted or escaped in any way. It's caller's job.
-
-=cut
-
-sub DateTimeIntervalFunction {
-    my $self = shift;
-    my %args = ( From => undef, To => undef, @_ );
-
-    $_ = DBIx::SearchBuilder->CombineFunctionWithField(%$_)
-        for grep ref, @args{'From', 'To'};
-
-    return $self->_DateTimeIntervalFunction( %args );
-}
-
-sub _DateTimeIntervalFunction { return 'NULL' }
-
-=head2 NullsOrder
-
-Sets order of NULLs when sorting columns when called with mode,
-but only if DB supports it. Modes:
-
-=over 4
-
-=item * small
-
-NULLs are smaller then anything else, so come first when order
-is ASC and last otherwise.
-
-=item * large
-
-NULLs are larger then anything else.
-
-=item * first
-
-NULLs are always first.
-
-=item * last
-
-NULLs are always last.
-
-=item * default
-
-Return back to DB's default behaviour.
-
-=back
-
-When called without argument returns metadata required to generate
-SQL.
-
-=cut
-
-sub NullsOrder {
-    my $self = shift;
-
-    unless ($self->HasSupportForNullsOrder) {
-        warn "No support for changing NULLs order" if @_;
-        return undef;
-    }
-
-    if ( @_ ) {
-        my $mode = shift || 'default';
-        if ( $mode eq 'default' ) {
-            delete $self->{'nulls_order'};
-        }
-        elsif ( $mode eq 'small' ) {
-            $self->{'nulls_order'} = { ASC => 'NULLS FIRST', DESC => 'NULLS LAST' };
-        }
-        elsif ( $mode eq 'large' ) {
-            $self->{'nulls_order'} = { ASC => 'NULLS LAST', DESC => 'NULLS FIRST' };
-        }
-        elsif ( $mode eq 'first' ) {
-            $self->{'nulls_order'} = { ASC => 'NULLS FIRST', DESC => 'NULLS FIRST' };
-        }
-        elsif ( $mode eq 'last' ) {
-            $self->{'nulls_order'} = { ASC => 'NULLS LAST', DESC => 'NULLS LAST' };
-        }
-        else {
-            warn "'$mode' is not supported NULLs ordering mode";
-            delete $self->{'nulls_order'};
-        }
-    }
-
-    return undef unless $self->{'nulls_order'};
-    return $self->{'nulls_order'};
-}
-
-=head2 HasSupportForNullsOrder
-
-Returns true value if DB supports adjusting NULLs order while sorting
-a column, for example C<ORDER BY Value ASC NULLS FIRST>.
-
-=cut
-
-sub HasSupportForNullsOrder {
-    return 0;
-}
-
 
 =head2 DESTROY
 
@@ -1723,6 +1600,7 @@ When we get rid of the Searchbuilder::Handle, we need to disconnect from the dat
 
 =cut
 
+  
 sub DESTROY {
   my $self = shift;
   $self->Disconnect if $self->{'DisconnectHandleOnDestroy'};
