@@ -69,9 +69,9 @@ sub SimpleUpdateFromSelect {
     return $sth unless $sth;
 
     my (@binds, @columns);
-    for my $k (sort keys %$values) {
+    while ( my ($k, $v) = each %$values ) {
         push @columns, $k;
-        push @binds, $values->{$k};
+        push @binds, $v;
     }
 
     my $update_query = "UPDATE $table SET "
